package org.apache.bcel.classfile;

import java.io.DataInput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Arrays;
import org.apache.commons.lang3.ArrayUtils;

public final class Code extends Attribute {
   private int maxStack;
   private int maxLocals;
   private byte[] code;
   private CodeException[] exceptionTable;
   private Attribute[] attributes;

   public Code(Code c) {
      this(c.getNameIndex(), c.getLength(), c.getMaxStack(), c.getMaxLocals(), c.getCode(), c.getExceptionTable(), c.getAttributes(), c.getConstantPool());
   }

   Code(int nameIndex, int length, DataInput file, ConstantPool constantPool) throws IOException {
      this(nameIndex, length, file.readUnsignedShort(), file.readUnsignedShort(), (byte[])null, (CodeException[])null, (Attribute[])null, constantPool);
      int codeLength = file.readInt();
      this.code = new byte[codeLength];
      file.readFully(this.code);
      int exceptionTableLength = file.readUnsignedShort();
      this.exceptionTable = new CodeException[exceptionTableLength];

      int attributesCount;
      for(attributesCount = 0; attributesCount < exceptionTableLength; ++attributesCount) {
         this.exceptionTable[attributesCount] = new CodeException(file);
      }

      attributesCount = file.readUnsignedShort();
      this.attributes = new Attribute[attributesCount];

      for(int i = 0; i < attributesCount; ++i) {
         this.attributes[i] = Attribute.readAttribute(file, constantPool);
      }

      super.setLength(length);
   }

   public Code(int nameIndex, int length, int maxStack, int maxLocals, byte[] code, CodeException[] exceptionTable, Attribute[] attributes, ConstantPool constantPool) {
      super((byte)2, nameIndex, length, constantPool);
      this.maxStack = maxStack;
      this.maxLocals = maxLocals;
      this.code = code != null ? code : ArrayUtils.EMPTY_BYTE_ARRAY;
      this.exceptionTable = exceptionTable != null ? exceptionTable : CodeException.EMPTY_CODE_EXCEPTION_ARRAY;
      this.attributes = attributes != null ? attributes : EMPTY_ARRAY;
      super.setLength(this.calculateLength());
   }

   public void accept(Visitor v) {
      v.visitCode(this);
   }

   private int calculateLength() {
      int len = 0;
      if (this.attributes != null) {
         Attribute[] var2 = this.attributes;
         int var3 = var2.length;

         for(int var4 = 0; var4 < var3; ++var4) {
            Attribute attribute = var2[var4];
            len += attribute.getLength() + 6;
         }
      }

      return len + this.getInternalLength();
   }

   public Attribute copy(ConstantPool constantPool) {
      Code c = (Code)this.clone();
      if (this.code != null) {
         c.code = (byte[])this.code.clone();
      }

      c.setConstantPool(constantPool);
      c.exceptionTable = new CodeException[this.exceptionTable.length];
      Arrays.setAll(c.exceptionTable, (i) -> {
         return this.exceptionTable[i].copy();
      });
      c.attributes = new Attribute[this.attributes.length];
      Arrays.setAll(c.attributes, (i) -> {
         return this.attributes[i].copy(constantPool);
      });
      return c;
   }

   public void dump(DataOutputStream file) throws IOException {
      super.dump(file);
      file.writeShort(this.maxStack);
      file.writeShort(this.maxLocals);
      file.writeInt(this.code.length);
      file.write(this.code, 0, this.code.length);
      file.writeShort(this.exceptionTable.length);
      CodeException[] var2 = this.exceptionTable;
      int var3 = var2.length;

      int var4;
      for(var4 = 0; var4 < var3; ++var4) {
         CodeException exception = var2[var4];
         exception.dump(file);
      }

      file.writeShort(this.attributes.length);
      Attribute[] var6 = this.attributes;
      var3 = var6.length;

      for(var4 = 0; var4 < var3; ++var4) {
         Attribute attribute = var6[var4];
         attribute.dump(file);
      }

   }

   public Attribute[] getAttributes() {
      return this.attributes;
   }

   public byte[] getCode() {
      return this.code;
   }

   public CodeException[] getExceptionTable() {
      return this.exceptionTable;
   }

   private int getInternalLength() {
      return 8 + this.code.length + 2 + 8 * (this.exceptionTable == null ? 0 : this.exceptionTable.length) + 2;
   }

   public LineNumberTable getLineNumberTable() {
      Attribute[] var1 = this.attributes;
      int var2 = var1.length;

      for(int var3 = 0; var3 < var2; ++var3) {
         Attribute attribute = var1[var3];
         if (attribute instanceof LineNumberTable) {
            return (LineNumberTable)attribute;
         }
      }

      return null;
   }

   public LocalVariableTable getLocalVariableTable() {
      Attribute[] var1 = this.attributes;
      int var2 = var1.length;

      for(int var3 = 0; var3 < var2; ++var3) {
         Attribute attribute = var1[var3];
         if (attribute instanceof LocalVariableTable) {
            return (LocalVariableTable)attribute;
         }
      }

      return null;
   }

   public int getMaxLocals() {
      return this.maxLocals;
   }

   public int getMaxStack() {
      return this.maxStack;
   }

   public void setAttributes(Attribute[] attributes) {
      this.attributes = attributes != null ? attributes : EMPTY_ARRAY;
      super.setLength(this.calculateLength());
   }

   public void setCode(byte[] code) {
      this.code = code != null ? code : ArrayUtils.EMPTY_BYTE_ARRAY;
      super.setLength(this.calculateLength());
   }

   public void setExceptionTable(CodeException[] exceptionTable) {
      this.exceptionTable = exceptionTable != null ? exceptionTable : CodeException.EMPTY_CODE_EXCEPTION_ARRAY;
      super.setLength(this.calculateLength());
   }

   public void setMaxLocals(int maxLocals) {
      this.maxLocals = maxLocals;
   }

   public void setMaxStack(int maxStack) {
      this.maxStack = maxStack;
   }

   public String toString() {
      return this.toString(true);
   }

   public String toString(boolean verbose) {
      StringBuilder buf = new StringBuilder(100);
      buf.append("Code(maxStack = ").append(this.maxStack).append(", maxLocals = ").append(this.maxLocals).append(", code_length = ").append(this.code.length).append(")\n").append(Utility.codeToString(this.code, super.getConstantPool(), 0, -1, verbose));
      int var4;
      int var5;
      if (this.exceptionTable.length > 0) {
         buf.append("\nException handler(s) = \n").append("From\tTo\tHandler\tType\n");
         CodeException[] var3 = this.exceptionTable;
         var4 = var3.length;

         for(var5 = 0; var5 < var4; ++var5) {
            CodeException exception = var3[var5];
            buf.append(exception.toString(super.getConstantPool(), verbose)).append("\n");
         }
      }

      if (this.attributes.length > 0) {
         buf.append("\nAttribute(s) = ");
         Attribute[] var7 = this.attributes;
         var4 = var7.length;

         for(var5 = 0; var5 < var4; ++var5) {
            Attribute attribute = var7[var5];
            buf.append("\n").append(attribute.getName()).append(":");
            buf.append("\n").append(attribute);
         }
      }

      return buf.toString();
   }
}
